<?php

use Elvez\AdminUtilV9 as AdminUtil;

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://elvez.co.jp
 * @since      1.0.0
 * @package    Elvez_Control_Access
 * @subpackage Elvez_Control_Access/admin
 * @author     Elvez, Inc. <info@elvez.co.jp>
 */
class Elvez_Control_Access_Admin {
	/**
	 * 翻訳ファイルのドメイン
	 */
	const TEXT_DOMAIN = Elvez_Control_Access::TEXT_DOMAIN;

	/**
	 * トップレベルメニューのスラグ
	 */
	const ELVEZ_TOP_LEVEL_ADMIN_MENU = 'elvez_admin';

	/**
	 * 設定メニューの権限
	 */
	const CAPABILITY = 'manage_options';

	/**
	 * 設定ページのURLスラグ
	 */
	const MENU_SLUG = 'elvez-control-access';

	/**
	 * 設定メニューのアイコン
	 * https://developer.wordpress.org/resource/dashicons/
	 */
	const ICON_URL = 'dashicons-edit-large';

	/**
	 * 設定メニューの表示位置
	 */
	const POSITION = 99;

	/**
	 * オプションのグループ名
	 */
	const OPTION_GROUP = 'elvez-control-access';

	/**
	 * オプションフィールド名のプレフィックス
	 */
	const OPTION_PREFIX = 'elvez_control_access_';

	/**
	 * WooCommerceのアクセス許可設定
	 * @since 1.0.5
	 */
	const OPTION_ACCESS_CONTROL_WOOCOMMERCE = self::OPTION_PREFIX . 'is_private_woocommerce';

	/**
	 * リダイレクト先の設定オプション
	 */
	const OPTION_REDIRECT_PAGE = self::OPTION_PREFIX . 'redirect_page';

	/**
	 * アクセス制御の投稿メタオプション
	 */
	const POST_META_ACCESS_CONTROL = self::OPTION_PREFIX . 'restrict_access';

	/**
	 * 投稿メタオプション設定フォームのセクションID
	 */
	const META_BOX_ID= 'elvez-control-access';

	/**
	 * 投稿メタオプション設定フォームのnonce
	 */
	const NONCE_KEY = 'elvez_control_access';

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

		add_action( 'admin_menu', [$this, 'register_meta_box'] );
		add_action( 'save_post', [$this, 'save_post_meta'] );

		add_filter( 'manage_posts_columns', [$this, 'show_control_status'] );
		add_filter( 'manage_pages_columns', [$this, 'show_control_status'] );
		add_action( 'manage_posts_custom_column' , [$this, 'custom_column'], 10, 2 );
		add_action( 'manage_pages_custom_column' , [$this, 'custom_column'], 10, 2 );

		add_action( 'admin_init', [$this, 'register_settings'] );
		add_action( 'admin_menu', [$this, 'set_menu_page'] );

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Elvez_Control_Access_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Elvez_Control_Access_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/elvez-control-access-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Elvez_Control_Access_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Elvez_Control_Access_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/elvez-control-access-admin.js', array( 'jquery' ), $this->version, false );

	}

	/**
	 * Register setting fields.
	 *
	 * @since    1.0.0
	 */
	public function register_settings() {
		/**
		 * Page Regirect
		 */
		add_option(self::OPTION_REDIRECT_PAGE, '');
		register_setting( self::OPTION_GROUP, self::OPTION_REDIRECT_PAGE);
		/**
		 * @since 1.0.5
		 * WooCommerce設定
		 */
		add_option(self::OPTION_ACCESS_CONTROL_WOOCOMMERCE, 0);
		register_setting( self::OPTION_GROUP, self::OPTION_ACCESS_CONTROL_WOOCOMMERCE);
		/**
		 * Subscriptions
		 */
		add_option(Elvez_Control_Access::OPTION_SUBSCRIBE_EMAIL, '');
		register_setting( self::OPTION_GROUP, Elvez_Control_Access::OPTION_SUBSCRIBE_EMAIL );
		add_option(Elvez_Control_Access::OPTION_SUBSCRIPTION_ID, '');
		register_setting( self::OPTION_GROUP, Elvez_Control_Access::OPTION_SUBSCRIPTION_ID );

	}


	/**
	 * Register custom metabox.
	 *
	 * @since     1.0.0
	 */
	function register_meta_box() {
		$id = self::META_BOX_ID;
		$title = __( 'Access Control', self::TEXT_DOMAIN );
		add_meta_box( $id, $title, [$this, 'render_meta_box'], 'post', 'side' );
		add_meta_box( $id, $title, [$this, 'render_meta_box'], 'page', 'side' );
	}

	/**
	 * Render metabox in edit page.
	 *
	 * @since     1.0.0
	 */
	function render_meta_box($post, $metabox) {
		$value = get_post_meta( $post->ID, self::POST_META_ACCESS_CONTROL, true );
		wp_nonce_field('wp-nonce-key', self::NONCE_KEY);
		?>
		<div>
			<label for="">
				<input type="checkbox"
					name="<?php echo esc_attr( self::POST_META_ACCESS_CONTROL ); ?>"
					value="1"
					<?php if ( 1 === intval( $value ) ) {
						echo 'checked="checked"';
					} ?>
				/>
				<?php esc_html_e( 'Allow only login user access', self::TEXT_DOMAIN ); ?>
			</label>
		</div>
		<?php
	}

	/**
	 * Save post meta params.
	 * nonceを検証
	 * nonceの検証に成功したらパラメータをチェック
	 * パラメータが存在するならsanitizeして更新
	 * パラメータが空ならメタデータを削除
	 *
	 * @since	1.0.0
	 * @params	$post_id	投稿ID
	 */
	function save_post_meta($post_id) {
		$meta_key = self::POST_META_ACCESS_CONTROL;
		$nonce = self::NONCE_KEY;

		if ( isset($_POST[$nonce]) && check_admin_referer('wp-nonce-key', $nonce) ) {
			if ( isset($_POST[$meta_key]) ) {
				$value = sanitize_meta( $meta_key, $_POST[$meta_key], 'post' );
				update_post_meta( $post_id, $meta_key, $value );
			} else {
				delete_post_meta( $post_id, $meta_key );
			}
		}
	}

	/**
	 * Show accsess control status in posts and pages list colmun.
	 *
	 * @since	1.0.0
	 * @params	$columns	array	元々のカラム一覧
	 * @return	$columns	array	表示するカラム一覧
	 */
	function show_control_status($columns) {
		$key = self::POST_META_ACCESS_CONTROL;
		$columns[$key] = __('Access Control', self::TEXT_DOMAIN);
		return $columns;
	}

	/**
	 * Rendre access control status.
	 *
	 * @since	1.0.0
	 * @params	$columns	array	元々のカラム一覧
	 * @return	$columns	array	表示するカラム一覧
	 */
	function custom_column($column, $post_id) {
		$key = self::POST_META_ACCESS_CONTROL;
		if ( $column == $key ) {
			$value = get_post_meta( $post_id , $key , true );
			$state = __( 'Allow only login user access', self::TEXT_DOMAIN );
			if ( $value ) {
				echo '<span class="dashicons dashicons-lock"></span><span>'. $state .'</span>';
			} else {
				echo '<span>—</span>';
			}
		}
	}

	/**
	 * Register top-level menu.
	 *
	 * @since    1.0.0
	 */
	public function register_top_level_menu() {
		global $menu;
		if ( !isset($menu[self::ELVEZ_TOP_LEVEL_ADMIN_MENU]) ) {
			// FIXME: Use icon image.
			// $icon = plugin_dir_url( __FILE__ ) . 'img/menu_icon.svg';

			add_menu_page(
				__( 'Elvez', self::TEXT_DOMAIN ),  // Page title
				__( 'Elvez', self::TEXT_DOMAIN ),  // Menu title
				self::CAPABILITY,
				self::ELVEZ_TOP_LEVEL_ADMIN_MENU,
				[$this, 'render_menu_page'], // Display first submenu page
				self::ICON_URL,
				self::POSITION
			);
		}
	}
	/**
	 * Set menu page.
	 *
	 * @since    1.0.0
	 */
	public function set_menu_page() {
		$this->register_top_level_menu();

		add_submenu_page(
			self::ELVEZ_TOP_LEVEL_ADMIN_MENU,
			__( 'Control Access', self::TEXT_DOMAIN ),  // Page title
			__( 'Control Access', self::TEXT_DOMAIN ),  // Menu title
			self::CAPABILITY,
			self::MENU_SLUG,
			[$this, 'render_menu_page'],  // Callback functions to render menu page
		);
		remove_submenu_page(self::ELVEZ_TOP_LEVEL_ADMIN_MENU, self::ELVEZ_TOP_LEVEL_ADMIN_MENU);
	}

	/**
	 * Re render menu page html.
	 *
	 * @since    1.0.0
	 */
	public function render_menu_page() {
		$logout_user_redirects = array(
			'selected'         => get_option( self::OPTION_REDIRECT_PAGE ),
			'name'             => self::OPTION_REDIRECT_PAGE,
			'show_option_none' => __( 'Return 404 not found', self::TEXT_DOMAIN ),
			'class' => 'elv-need-subscribe',
		);

		?>
		<div class="wrap">
			<h1 class=""><?php esc_html_e( 'Settings for Elvez Control Access' , self::TEXT_DOMAIN ); ?></h1>

			<form method="post" action="options.php">
				<?php settings_fields(self::OPTION_GROUP); ?>
				<?php do_settings_sections(self::OPTION_GROUP); ?>

				<!-- Subscription Settings -->
				<?php Elvez_Control_Access::render_subscription_form() ?>

				<table class="form-table">
					<tr valign="top">
						<th scope="row"><?php esc_html_e( 'Logout User Redirect', self::TEXT_DOMAIN ); ?></th>
						<td><?php wp_dropdown_pages($logout_user_redirects) ?></td>
					</tr>
				</table>

				<table class="form-table">
					<tr valign="top">
						<th scope="row"><?php esc_html_e( 'Enable Access Control to WooCommerce', self::TEXT_DOMAIN ); ?></th>
						<td><input type="checkbox"
							class="elv-need-subscribe" disabled
							name="<?php echo esc_attr( self::OPTION_ACCESS_CONTROL_WOOCOMMERCE ); ?>"
							value="1"
							<?php if ( 1 === intval( get_option( self::OPTION_ACCESS_CONTROL_WOOCOMMERCE ) ) ) {
								echo 'checked="checked"';
							} ?>
						/>
						<p class="description"><?php esc_html_e( 'Control accesses to shop, products, cart, and checkout page.', self::TEXT_DOMAIN ); ?></p>
						<p class="description"><?php esc_html_e( 'To exclude from the search, you can set the access control for each page.', self::TEXT_DOMAIN ); ?></p>
						</td>
					</tr>
				</table>

				<?php submit_button(); ?>
			</form>
		</div>
		<?php
	}

}
