<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://elvez.co.jp
 * @since      1.0.0
 *
 * @package    Elvez_Control_Access
 * @subpackage Elvez_Control_Access/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Elvez_Control_Access
 * @subpackage Elvez_Control_Access/public
 * @author     Elvez, Inc. <info@elvez.co.jp>
 */
class Elvez_Control_Access_Public {
	/**
	 * @since	1.1.3
	 */
	const FILTER_IS_RESTRICT = 'elv_control_access_is_restrict';

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

		add_action( 'template_redirect', [$this, 'control_access'] );
		add_action( 'wp', [$this, 'control_access'] );

		add_action( 'pre_get_posts', [$this, 'filter_hidden_posts'] );
		add_filter( 'get_pages', [$this, 'filter_hidden_pages'] );
		add_filter( 'wp_get_nav_menu_items',   [$this, 'filter_hidden_nav_items'], null, 3 );
		add_filter( 'get_previous_post_where', [$this, 'filter_hidden_nex_prev_link'] );
		add_filter( 'get_next_post_where', [$this, 'filter_hidden_nex_prev_link'] );

		add_action( 'init', [$this, 'register_dependencies'] );
	}

	/**
	 * Get registered version of script or style.
	 * 指定キーのスクリプトまたはスタイルが登録されていたらそのバージョン番号を返す
	 * 登録されていない場合は '0' を返す
	 *
	 * @since     1.0.4
	 * @param	$key	string
	 * @param	$type	string	'scripts' | 'styles'
	 * @return	$ver	string	Version string
	 */
	public static function get_registered_version( $key, $type='scripts') {

		$not_registerd_ver = '0';

		if ( $type == 'scripts' ) {
			$dependencies = wp_scripts();
		} else if ( $type == 'styles' ) {
			$dependencies = wp_styles();
		} else {
			return $not_registerd_ver;
		}

		if ( isset( $dependencies->registered[$key] ) ) {
			$registerd = $dependencies->registered[$key];
			return $registerd->ver;
		} else {
			return $not_registerd_ver;
		}

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.4
	 */
	public function register_dependencies() {

		$elvez_style_version = '1.1.7';
		$style_registered_ver = $this->get_registered_version( 'elvez-style', 'styles' );
		if ( version_compare( $elvez_style_version, $style_registered_ver, '>' ) ) {
			wp_styles()->remove( 'elvez-style' );
			wp_register_style( 'elvez-style', plugin_dir_url( __FILE__ ) . 'css/elvez-style.css', array(), $elvez_style_version, 'all' );

		}

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		wp_enqueue_style( 'elvez-style' );

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/elvez-control-access-public.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/elvez-control-access-public.js', array( 'jquery' ), $this->version, false );

	}

	/**
	 * Redirect access on restrict user in controled posts.
	 * 未ログインかつ制御対象の投稿であればリダイレクト
	 * リダイレクト条件
	 * 	ログインしていない　&& ( 制御対象投稿かつ単体ページ || Woocommerceオプション有効かつWoocmmerceページ )
	 * リダイレクト先が未設定なら404を返す
	 * 現在の投稿とリダイレクト先が同一であれば404を返す（無限リダイレクト防止）
	 *
	 * @since	1.0.0
	 * @since	1.0.5 check woocommerce condition.
	 * @since	1.1.0 Redirect to page only subscribed
	 */
	public function control_access() {
		$post_id = get_the_ID();
		$is_restrict_singular = $this->get_restrict_singular( $post_id );
		$is_restrict_woocommerce = $this->get_restrict_woocommerce();

		//$is_redirect = !is_user_logged_in() && $controled && is_singular;
		$is_restrict = !is_user_logged_in() && ( $is_restrict_singular || $is_restrict_woocommerce );
		$is_restrict = apply_filters( static::FILTER_IS_RESTRICT, $is_restrict );

		if ( $is_restrict ) {
			$is_subscribed = Elvez_Control_Access::is_subscribed();
			$redirect_page = get_option( Elvez_Control_Access_Admin::OPTION_REDIRECT_PAGE );
			if ( $is_subscribed && $redirect_page && $redirect_page != $post_id ) {
				$redirect_url = get_permalink($redirect_page);
				wp_redirect( $redirect_url );
				exit;
			} else {
				global $wp_query;
				$wp_query->set_404();
				status_header(404);
			}
		}
	}
	/**
	 * Return restrict woocommerce condition.
	 *
	 * WooCommerceオプションが有効かつWooCommerceプラグインが有効かつWoocommerceページである場合、trueを返す
	 *
	 * @since	1.0.5
	 * @param	$post_id
	 * @return	bool
	 */
	public function get_restrict_singular( $post_id ) {
		$meta_name = Elvez_Control_Access_Admin::POST_META_ACCESS_CONTROL;
		$is_controled = get_post_meta( $post_id, $meta_name, true );
		$is_singular = is_singular();
		return $is_controled && $is_singular;
	}
	/**
	 * Return restrict woocommerce condition.
	 *
	 * WooCommerceオプションが有効かつWooCommerceプラグインが有効かつWoocommerceページである場合、trueを返す
	 * カート、チェックアウトページはis_woocommerceがfalseになるので、別途判定。
	 *
	 * @since	1.0.5
	 * @since	1.1.0	Return false if not subscribed.
	 * @return	bool
	 */
	public function get_restrict_woocommerce() {
		$is_subscribed = Elvez_Control_Access::is_subscribed();
		if ( !$is_subscribed ) {
			return false;
		}
		$restrict_woocommerce = get_option( Elvez_Control_Access_Admin::OPTION_ACCESS_CONTROL_WOOCOMMERCE, null );
		$is_woocommerce = function_exists( 'is_woocommerce' ) && is_woocommerce();
		$is_cart = function_exists( 'is_cart' ) && is_cart();
		$is_checkout = function_exists( 'is_checkout' ) && is_checkout();
		return $restrict_woocommerce && ( $is_woocommerce || $is_cart || $is_checkout );
	}

	/**
	 * Return controled posts and pages.
	 *
	 * @since	1.0.1
	 * return	array	$hidden_posts	非表示にする投稿ID一覧
	 */
	public function get_hidden_posts() {
		global $wpdb;
		$hidden_posts = array();

		$key = Elvez_Control_Access_Admin::POST_META_ACCESS_CONTROL;
		$value = '1';
		$post_table = $wpdb->posts;
		$meta_table = $wpdb->postmeta;

		$sql = "SELECT post_id FROM $meta_table WHERE (meta_key = '$key' AND meta_value = '$value')";

		$results = $wpdb->get_results( $sql );

		foreach( $results as $result ) {
			$hidden_posts[] = $result->post_id;
		}
		return $hidden_posts;
	}

	/**
	 * Filter hidden posts in query.
	 * パーマリンクにslugを使用している場合、メインクエリで表示対象の記事をフィルタしてしまうと記事IDが取得できない
	 * メインクエリではフィルタしない
	 * 検索の場合はメインクエリでもフィルタする
	 * アーカイブページ（カテゴリ、タグ、月別等）の場合もフィルタする
	 * ホームの「最新の投稿」はis_archive=falseのため、is_home=trueでもフィルタする
	 *
	 * @since 1.0.1
	 *
	 * @param  WP_Query	$query	投稿取得のクエリ
	 * @return WP_Query $query	post__not_inを更新したクエリ
	 */
	function filter_hidden_posts( $query ) {
		$user = wp_get_current_user();
		$is_user_logged_in = $user->ID != 0 or $user->ID != null;
		if ( !$is_user_logged_in ) {
			if ( !$query->is_main_query() || $query->is_search() || $query->is_archive() || $query->is_home() ) {
					$hidden_posts = $this->get_hidden_posts();
				if ( !empty( $hidden_posts ) ) {
					$not_in = array_merge( $query->query_vars['post__not_in'], $hidden_posts );
					$query->set( 'post__not_in', $not_in );
				}
			}
		}
		return $query;
	}

	/**
	 * Filter hidden posts for get_pages().
	 *
	 * @since 1.0.2
	 *
	 * @param	array	$pages
	 * @return	array	$pages
	 */
	function filter_hidden_pages( $pages ) {
		if ( !is_user_logged_in() ) {
			$hidden_posts = $this->get_hidden_posts();
			if ( !empty( $hidden_posts ) ) {
				foreach( $pages as $key => $page ) {
					if ( in_array( $page->ID, $hidden_posts ) ) {
						unset( $pages[ $key ] );
					}

				}
			}
		}
		return $pages;
	}

	/**
	 * Filter hidden posts in navigation menu items.
	 *
	 * @since 1.0.2
	 *
	 * @param	array	$items
	 * @param 	object	$menu
	 * @param 	array	$args
	 * @return	array	$items
	 */
	function filter_hidden_nav_items( $items, $menu, $args ) {
		if ( !is_user_logged_in() ) {
			$hidden_posts = $this->get_hidden_posts();
			if ( !empty( $hidden_posts ) ) {
				foreach( $items as $key => $item ) {
					if ( in_array( $item->object_id, $hidden_posts ) ) {
						unset( $items[ $key ] );
					}
				}
			}
		}
		return $items;
	}

	/**
	 * Filter hidden posts on prev/next links in singlar post.
	 *
	 * @since 1.0.2
	 *
	 * @param  string $where
	 * @return string $where
	 */
	function filter_hidden_nex_prev_link( $where ) {
		if ( ! is_user_logged_in() ) {
			$hidden_posts = $this->get_hidden_posts();
			if ( !empty( $hidden_posts ) ) {
				$str = implode( ',', $hidden_posts );
				$where  = "$where AND p.ID NOT IN ( $str )";
			}
		}
		return $where;
	}

}
